/// =============================================================================
// Stereo recognition - determining depth in scene
// Image Processing and Computer Vision (COMS 30121), assignment 2
//
// author: Jaroslav Borovicka, December 2002 - January 2003
//         University of Bristol
//         Czech Technical University in Prague
// =============================================================================
// matrices.cpp
// header file for the matrix and vector classes
// =============================================================================

// This small library defines the classes for vector and matrix calculus
// (i.e. data structures and operators) required for the assignment.
//
// vector, matrix - float classes
// ivector, imatrix - integer classes 

#ifndef __matrices
#define __matrices

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

typedef long unsigned int UINT32;
typedef long int          INT32;

class vector
{

protected:
  int m;
  float *data;

public:
  vector();
  vector(int a);
  ~vector();

  friend vector operator *(float c, vector v);
  friend float operator *(vector m1, vector m2);  // scalar multiplication
  friend vector operator ^(vector u, vector v);  // vector multiplication
  friend vector operator +(vector m1, vector m2);
  friend vector operator -(vector m1, vector m2);
  vector operator-();

  float& operator [](int i) { return data[i-1]; };
  const float& operator [](int i) const { return data[i-1]; };

  vector (const vector &mat);
  vector& operator =(const vector &m);

  friend int operator==(vector v1, vector v2);
  friend int operator!=(vector v1, vector v2);
//  vector operator =(float *content) { data = content; return *this;};

//  vector operator =(float *content) { data = content; return *this;};
  void setdata(float *content, int l);

  int length() { return m; };
  float abs();

  void print();

};

typedef vector *Pvector;

// =============================================================================
// class matrixrow

class matrixrow : public vector
{

protected:
  int m;
  float *data;

public:
  matrixrow() { data = NULL; m = 0; };
  matrixrow(float *d, int l) { data = d; m = l; };
  ~matrixrow() { data = NULL; };

//  friend float operator *(vector m1, vector m2);
  float& operator[](int i) { return data[i-1]; };
  const float& operator[](int i) const { return data[i-1]; };

  matrixrow operator =(vector v);
  matrixrow operator =(matrixrow v);

  matrixrow (const matrixrow &mr);  // copy constructor

};

typedef matrixrow *Pmatrixrow;


// =============================================================================
// matrix

class matrix : public vector
{
  int m, n;  // # of rows and columns
  float **data;

public:
  matrix();
  matrix(int a, int b);
  ~matrix();

  friend matrix operator *(float c, matrix mat);
  friend matrix operator *(matrix m1, matrix m2);
  friend vector operator *(matrix mat, vector vect);
  friend vector operator *(vector vect, matrix mat);

  friend matrix operator +(matrix m1, matrix m2);
  friend matrix operator -(matrix m1, matrix m2);

  //  float *operator[](int i) { return data[i-1]; };
//  const float *operator[](int i) const { return data[i-1]; };

  matrixrow operator[](int i) {
    matrixrow r(data[i-1], n);
    return r;
  };

  const matrixrow operator[](int i) const {
    matrixrow r(data[i-1], n);
    return r;
  };

  matrix (const matrix &mat);
  matrix& operator =(const matrix &m);

  const int rows() const { return m; };
  const int columns() const { return n; };

  void setdata(float **content, int a, int b);

  matrix row(int i);
  matrix column(int i);

  vector vrow(int i);
  const vector vrow(int i) const;
  vector vcolumn(int i);

  matrix transpose ();
  matrix inverse ();
  void print();
};

typedef matrix *Pmatrix;


// =============================================================================
// ivector

class ivector
{

protected:
  int m;
  INT32 *data;

public:
  ivector();
  ivector(int a);
  ~ivector();

  friend ivector operator *(INT32 c, ivector v);
  friend INT32 operator *(ivector m1, ivector m2);  // scalar multiplication
  friend ivector operator ^(ivector u, ivector v);  // vector multiplication
  friend ivector operator +(ivector m1, ivector m2);
  friend ivector operator -(ivector m1, ivector m2);
  INT32& operator [](int i) { return data[i-1]; };
  const INT32& operator [](int i) const { return data[i-1]; };

  ivector (const ivector &mat);
  ivector& operator =(const ivector &m);

  friend int operator &(ivector v1, ivector v2);
  ivector operator-();

  friend int operator==(ivector v1, ivector v2);
  friend int operator!=(ivector v1, ivector v2);
//  vector operator =(float *content) { data = content; return *this;};

//  vector operator =(float *content) { data = content; return *this;};
  void setdata(INT32 *content, int l);

  int length() { return m; };
  INT32 abssq();

  void print();

};

typedef ivector *Pivector;

// =============================================================================
// class imatrixrow

class imatrixrow : public ivector
{

protected:
  int m;
  INT32 *data;

public:
  imatrixrow() { data = NULL; m = 0; };
  imatrixrow(INT32 *d, int l) { data = d; m = l; };
  ~imatrixrow() { data = NULL; };

//  friend float operator *(vector m1, vector m2);
  INT32& operator[](int i) { return data[i-1]; };
  const INT32& operator[](int i) const { return data[i-1]; };
  imatrixrow operator =(ivector v);
  imatrixrow operator =(imatrixrow v);

  imatrixrow (const imatrixrow &mr);  // copy constructor

};

typedef imatrixrow *Pimatrixrow;


// =============================================================================
// imatrix

class imatrix : public ivector
{
  int m, n;  // # of rows and columns
  INT32 **data;

public:
  imatrix();
  imatrix(int a, int b);
  ~imatrix();

  friend imatrix operator *(imatrix m1, imatrix m2);
  friend ivector operator *(imatrix mat, ivector vect);
  friend ivector operator *(ivector vect, imatrix mat);

  friend imatrix operator +(imatrix m1, imatrix m2);
  friend imatrix operator -(imatrix m1, imatrix m2);

  //  float *operator[](int i) { return data[i-1]; };
//  const float *operator[](int i) const { return data[i-1]; };

  imatrixrow operator[](int i) {
    imatrixrow r(data[i-1], n);
    return r;
  };

  const imatrixrow operator[](int i) const {
    imatrixrow r(data[i-1], n);
    return r;
  };

  imatrix (const imatrix &mat);
  imatrix& operator =(const imatrix &m);

  const int rows() const { return m; };
  const int columns() const { return n; };

  void setdata(INT32 **content, int a, int b);

  imatrix row(int i);
  imatrix column(int i);

  ivector vrow(int i);
  const ivector vrow(int i) const;
  ivector vcolumn(int i);

  imatrix transpose ();
//  matrix inverse ();
  void print();
};

typedef imatrix *Pimatrix;


#endif