// =============================================================================
// Stereo recognition - determining depth in scene
// Image Processing and Computer Vision (COMS 30121), assignment 2
//
// author: Jaroslav Borovicka, April - May 2003
//         University of Bristol
//         Czech Technical University in Prague
// =============================================================================
// stereo.h
// header for the main program and interface
// =============================================================================

#ifndef __stereo
#define __stereo

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>

#include "imagelib/iplib.h"
#include "matrices.h"

typedef struct
{
  int use2ndderivative;    // should the 2nd derivative filter be used?
  int refinecoordinates;   // should a final refinement of coordinates be used?

  char *scenename;         // pointer to the name of the scene
  int contthreshold;       // edge map threshold
  int smoothimage;         // size of smoothing filter
  int saveall;             // should all intermediate files be saved?
  int swapx, swapy;        // multiply x, y coordinates by -1 to flip view
  int restrictcoordinates;
  float x1, y1, z1, x2, y2, z2; // restrictions on the 3D coordinates
  int pointsgiven;         // are coordinates of the edge points loaded from a file
  char *pointsfile;        // file containing the edge points coordinates
  int connectpoints;       // should the points be connected with lines?
  int lookaround;          // how many pixels around the epipolar line
                           // should be examined
  int corrarea;            // size of correlation window (2*corrarea + 1);
  float corrthreshold;       // threshold for correlation coefficient
  int errorallowed;        // pixel error allowed between left-right and
                           // right-left match
  int showcorrs;

} TParams;

typedef TParams *PParams;

typedef struct Correspondence
{
  float initx, inity;       // initial values for the epipolar line
  float corr;               // correlation coefficient calculated for the match
  float xli, yli, xri, yri; // image pixel coordinates
  float xl, yl, xr, yr;     // image coordinates
  float X, Y, Z;            // scene coordinates
  float diff;               // distance between the two rays
  int valid;
  Correspondence *next;
} TCorrespondence;

typedef TCorrespondence *PCorrespondence;

typedef struct Connection
{
  int i1, j1, i2, j2;      // points in the image which are supposed
                           // to be connected
  Connection *next;
} TConnection;

typedef TConnection *PConnection;


class TPGMImage
{

public:
  char *Filename;
  int w, h;                     // width and height of the image
  unsigned char **Data;         // data in chars
  float **fData;                // data in floats
  unsigned char **EdgeMap;      // edge map
  float **fEdgeMap;             // edge map in floats
  unsigned char **DirectionMap; // direction map
  float **fDirectionMap;        // direction map in floats

  char index;

  TPGMImage(char *f, char c);
  ~TPGMImage();
  void GetEdgeMap (PParams Params);
  void EdgesUsing2ndDer (PParams P);

};
typedef TPGMImage *PPGMImage;

class TScene
{

public:
  char *Name;

  PPGMImage LImage, RImage;
  char *CalFilename;
  CalPars CParams;

  matrix F;    // fundamental matrix
  matrix E;    // essential matrix
  matrix R, S; // parameter matrices saved as matrix objects
  vector T;    // parameter T saved as vector object
  matrix MLinv, MRinv;

  PCorrespondence Cfirst, Ccurrent;       // pointers to the correspondences
  PConnection ConF, ConC;                 // pointers to the connections
                                          // between the points
  PCorrespondence **CorrespondenceMap;

  TScene (char *n);
  ~TScene ();
  void FindCorrespondences (PParams Params);
  void Triangulate (PParams Params);
  void SaveCoordinates (PParams Params);
  void ExaminePoint (int i, int j, PCorrespondence *Cor,
                     vector pOrigi, vector pTargi, matrix F,
                     PPGMImage ISource, PPGMImage ITarget,
                     PParams Params);

};
typedef TScene *PScene;

PParams ProcessParameters(int argc, char* argv[]);

void writefloatpgm (float **im, int w, int h, char *fname, float shift);

float **convolution (float **image, int iw, int ih,
                     float **filter, int fw, int fh);

float **smoothimage (float **im, int w, int h, PParams P);



#else

#endif
